@@ -48,7 +48,7 @@ my %mimeType = (
 %Image::ExifTool::iWork::Main = (
     GROUPS => { 0 => 'XML', 1 => 'XML', 2 => 'Document' },
     PROCESS_PROC => \&Image::ExifTool::XMP::ProcessXMP,
-    VARS => { NO_ID => 1 },
+    VARS => { ID_FMT => 'none' },
     NOTES => q{
         The Apple iWork '09 file format is a ZIP archive containing XML files
         similar to the Office Open XML (OOXML) format.  Metadata tags in iWork
@@ -215,7 +215,7 @@ information from Apple iWork '09 XML+ZIP files.
 
 =head1 AUTHOR
 
-Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
